#!/usr/bin/env python3

"""Plots outermost contours and max-speed contours.

Red for anti-cyclones, blue for cyclones. Squares for a well-defined
but invalid outermost contour. Crosses for a valid outermost contour
but with no distinct max-speed contour. Filled circles for a valid
outermost contour with a distinct max-speed contour.

"""

import argparse
import shapefile
import numpy as np
import matplotlib.pyplot as plt
import netCDF4
from mpl_toolkits import basemap
import math
import itertools

parser = argparse.ArgumentParser()
parser.add_argument("-v", "--velocity", help = "plot velocity field",
                    action = "store_true")
parser.add_argument("-s", "--scale", default = 20,
                    help = "scale of arrows for the velocity field")
args = parser.parse_args()

with netCDF4.Dataset("h.nc") as f:
    longitude = f.variables["lon"][:]
    latitude = f.variables["lat"][:]
    
lon_2d, lat_2d = np.meshgrid(longitude, latitude)
llcrnrlon = math.floor(longitude[0])
llcrnrlat = math.floor(latitude[0])
urcrnrlon = math.ceil(longitude[-1])
urcrnrlat = math.ceil(latitude[-1])
m = basemap.Basemap(llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat,
                    resolution = "l")

plt.figure()
m.plot(lon_2d.reshape(-1), lat_2d.reshape(-1), latlon = True, marker = "+",
       color = "gray", linestyle = "None")

reader_extr = shapefile.Reader("extremum_1")
reader_outer = shapefile.Reader("outermost_contour_1")

try:
    reader_m_s = shapefile.Reader("max_speed_contour_1")
except shapefile.ShapefileException:
    print("Shapefile max_speed_contour_1 not found. "
          "Max-speed contours will not be plotted.")
    w = shapefile.Writer(shapeType = shapefile.NULL)
    m_s_iterShapes = itertools.repeat(w)
else:
    m_s_iterShapes = reader_m_s.iterShapes()

for shape_rec_extr, shape_outer, shape_m_s \
    in zip(reader_extr.iterShapeRecords(), reader_outer.iterShapes(),
           m_s_iterShapes):
    points = shape_rec_extr.shape.points[0]
    
    if shape_rec_extr.record[4] == 0:
            # Anti-cyclone
        color = "red"
    else:
        color = "blue"
        
    m.plot(points[0], points[1], latlon = True, marker = "o", markersize = 10,
           color = color, fillstyle = "none")
    projected_points = m(*points)
    plt.annotate(str(shape_rec_extr.record[2]), projected_points,
                 xytext = (3, 3), textcoords = "offset points")

    if shape_outer.shapeType != shapefile.NULL:
        points = np.array(shape_outer.points)
        lines = m.plot(points[:, 0], points[:, 1], latlon = True, color = color)

        if shape_rec_extr.record[5] == 0:
            # Invalid outermost contour
            lines[0].set_marker("s")
            lines[0].set_fillstyle("none")
        elif shape_m_s.shapeType == shapefile.NULL:
            lines[0].set_marker("x")
        else:
            lines[0].set_marker("o")

        if shape_m_s.shapeType != shapefile.NULL:
            points = np.array(shape_m_s.points)
            m.plot(points[:, 0], points[:, 1], latlon = True, color = color,
                   marker = "o")

if args.velocity:
    with netCDF4.Dataset("uv.nc") as f:
        u_rotated, v_rotated = m.rotate_vector(f["u"][0], f["v"][0], longitude,
                                               latitude)
        
    quiver_return = m.quiver(lon_2d, lat_2d, u_rotated, v_rotated,
                             latlon = True, scale = args.scale,
                             scale_units = "width")
    plt.quiverkey(quiver_return, 0.75, 0.9, 1, r"1 m s$^{-1}$",
              coordinates = "figure")

m.drawmeridians(np.linspace(llcrnrlon, urcrnrlon, 6), labels = [0, 0, 0, 1])
m.drawparallels(np.linspace(llcrnrlat, urcrnrlat, 6), labels = [1, 0, 0, 0])
m.drawcoastlines()

plt.show()
