program test_get_snapshot

  use, intrinsic:: ieee_arithmetic, only: IEEE_SUPPORT_DATATYPE, &
       ieee_support_nan

  ! Libraries:
  use jumble, only: new_unit
  use netcdf, only: nf90_nowrite
  use netcdf95, only: nf95_open, find_coord, nf95_inquire_dimension, &
       nf95_get_var, nf95_close
  use nr_util, only: assert, deg_to_rad, twopi, pi
  use shapelib, only: shpfileobject, shpclose
  
  use derived_types, only: snapshot
  use dispatch_snapshot_m, only: dispatch_snapshot
  use get_snapshot_m, only: get_snapshot
  use init_shapefiles_m, only: init_shapefiles

  implicit none

  type(snapshot) s
  TYPE(shpfileobject) hshp_extremum ! shapefile extremum_$m
  TYPE(shpfileobject) hshp_outermost ! shapefile outermost_contour_$m
  TYPE(shpfileobject) hshp_max_speed ! shapefile max_speed_contour_$m

  integer unit_isolated, unit_number_eddies

  real:: min_amp = 0.
  ! minimum amplitude of ssh, between extremum and outermost contour,
  ! in m

  integer ncid, dimid, varid, nlon, nlat
  real lon_min, lon_max, lat_min, lat_max ! longitudes and latitudes, in degrees
  real step(2) ! longitude and latitude steps, in rad
  logical periodic ! grid is periodic in longitude

  integer, parameter:: max_radius(2) = [20, 20]
  ! maximum radius of an eddy in longitude and latitude, in number of
  ! grid points

  !--------------------------------------------------------------

  call assert(IEEE_SUPPORT_DATATYPE(0.), ieee_support_nan(0.), &
       "test_get_snapshot: not enough IEEE support")

  print *, "min_amp = ? "
  read *, min_amp
  print *, "min_amp = ", min_amp

  call nf95_open("h.nc", nf90_nowrite, ncid)

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "longitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlon)
  call nf95_get_var(ncid, varid, lon_min)
  call nf95_get_var(ncid, varid, lon_max, start = [nlon])

  call find_coord(ncid, dimid = dimid, varid = varid, std_name = "latitude")
  call nf95_inquire_dimension(ncid, dimid, nclen = nlat)
  call nf95_get_var(ncid, varid, lat_min)
  call nf95_get_var(ncid, varid, lat_max, start = [nlat])

  call nf95_close(ncid)
  
  call assert(lon_max > lon_min .and. lat_max > lat_min, &
       "test_get_snapshot coordinate order")
  step = [(lon_max - lon_min) / (nlon - 1), (lat_max - lat_min) / (nlat - 1)]  &
       * deg_to_rad

  ! As we are requiring the grid spacing to be uniform, the value of
  ! "periodic" must be consistent with the values of step(1) and nlon:
  periodic = nint(twopi / step(1)) == nlon
  print *, "periodic = ", periodic
  call assert(2 * max_radius(1) * step(1) < pi, "test_get_snapshot max_radius")

  call get_snapshot(s, min_amp, m = 1, n_proc = 1, k_end = 1, max_delta = 4, &
       nlon = nlon, nlat = nlat, k = 1, max_radius = max_radius, &
       corner = [lon_min, lat_min] * deg_to_rad, step = step, &
       periodic = periodic)

  call init_shapefiles(hshp_extremum, hshp_outermost, hshp_max_speed)
  
  call new_unit(unit_isolated)
  open(unit_isolated, file = "isolated_nodes_1.csv", status = "replace", &
       action = "write")
  write(unit_isolated, fmt = *) '"date index" "eddy index"' ! title line

  call new_unit(unit_number_eddies)
  open(unit_number_eddies, file = "number_eddies_1.csv", status = "replace", &
       action = "write")
  write(unit_number_eddies, fmt = *) '"date index" ' &
       // '"number of visible eddies" "number of interpolated eddies"'
  ! (title line)

  call dispatch_snapshot(s, hshp_extremum, hshp_outermost, hshp_max_speed, &
       unit_isolated, unit_number_eddies, k = 1, m = 1, k_begin = 1, &
       max_delta = 4)

  CALL shpclose(hshp_extremum)
  print *, 'Created shapefile "extremum_1".'
  CALL shpclose(hshp_outermost)
  print *, 'Created shapefile "outermost_contour_1".'
  CALL shpclose(hshp_max_speed)
  print *, 'Created shapefile "max_speed_contour_1".'
  close(unit_isolated)
  print *, 'Created "isolated_nodes_1.csv".'
  close(unit_number_eddies)
  print *, 'Created "number_eddies_1.csv".'

  if (s%number_vis_eddies /= 0) then
     print *, "Average number of points per outermost contour: ", &
          sum(s%list_vis%out_cont%n_points) / real(s%number_vis_eddies)
     print *, "Average number of points per max-speed contour: ", &
          sum(s%list_vis%speed_cont%n_points) / real(s%number_vis_eddies)
  else
     print *, "No visible eddy."
  end if

end program test_get_snapshot
