program test_local_extrema

  ! Reads ADT from a NetCDF file and writes the list of extrema to a
  ! CSV file. Also creates a NetCDF file containing the map of extrema.

  ! Note: the grid spacing does not need to be uniform.

  ! Libraries:
  use jumble, only: new_unit, get_command_arg_dyn
  use netcdf, only: nf90_nowrite, nf90_clobber, NF90_FLOAT, NF90_INT
  use netcdf95, only: nf95_open, nf95_close, nf95_inq_varid, nf95_get_var, &
       nf95_put_var, nf95_create, nf95_def_dim, nf95_def_var, nf95_put_att, &
       nf95_enddef, nf95_gw_var, nf95_get_att

  use local_extrema_m, only: local_extrema

  implicit none

  integer nlon, nlat
  integer ncid, varid, dimid_lat, dimid_lon
  integer varid_lat, varid_lon, varid_extr_map
  integer i, unit
  character(len = :), allocatable:: filename
  real, allocatable:: longitude(:) ! (nlon) in degrees
  real, allocatable:: latitude(:) ! (nlat) in degrees

  real, allocatable:: ssh(:, :) ! (1 - copy:nlon + copy, nlat)
  ! sea-surface height, in m
  
  integer, allocatable:: extr_map(:, :) ! (1 - copy:nlon + copy, nlat)
  ! map of extrema

  integer, allocatable:: ind_extr(:, :) ! (2, n_extr)
  ! indices in the two dimensions of extrema

  real, allocatable:: innermost_level(:) ! (n_extr)
  ! level of innermost contour, for each extremum

  logical, allocatable:: cyclone(:) ! (n_extr)
  real Fill_Value
  logical periodic ! in longitude
  integer copy

  !---------------------------------------------------------------------

  call get_command_arg_dyn(1, filename, message = "Required argument: ADT file")
  print *, "periodic = ? "
  read *, periodic
  print *, "periodic = ", periodic
  print *, "Reading from ", filename, "..."
  call nf95_open(filename, nf90_nowrite, ncid)

  call nf95_inq_varid(ncid, "lon", varid)
  call nf95_gw_var(ncid, varid, longitude)

  call nf95_inq_varid(ncid, "lat", varid)
  call nf95_gw_var(ncid, varid, latitude)

  nlon = size(longitude)
  nlat = size(latitude)
  copy = merge(1, 0, periodic)
  allocate(ssh(1 - copy:nlon + copy, nlat), &
       extr_map(1 - copy:nlon + copy, nlat))

  call nf95_inq_varid(ncid, "adt", varid)
  call nf95_get_var(ncid, varid, ssh(1:nlon, :))
  call nf95_get_att(ncid, varid, "_FillValue", Fill_Value)
  where (ssh(1:nlon, :) == Fill_Value) ssh(1:nlon, :) = huge(0.)

  if (periodic) then
     ssh(0, :) = ssh(nlon, :)
     ssh(nlon + 1, :) = ssh(1, :)
  end if

  call nf95_close(ncid)

  call local_extrema(extr_map, ind_extr, innermost_level, cyclone, ssh, &
       periodic, my_lbound = [1 - copy, 1])

  call new_unit(unit)
  open(unit, file = "test_local_extrema.csv", status = "replace", &
       action = "write")

  ! Title lines:
  write(unit, fmt = *) '"longitude index" "latitude index"'
  write(unit, fmt = *) '"" "" "" "degrees east" "degrees north" "m" "m"'
  write(unit, fmt = *) 'ilon ilat cyclone longitude latitude ssh ', &
       'innermost_level'

  do i = 1, size(ind_extr, 2)
     write(unit, fmt = *) ind_extr(1, i), ind_extr(2, i), cyclone(i), &
          longitude(ind_extr(1, i)), latitude(ind_extr(2, i)), &
          ssh(ind_extr(1, i), ind_extr(2, i)), innermost_level(i)
  end do
  close(unit)
  print *, 'Created file "test_local_extrema.csv".'

  call nf95_create("test_local_extrema.nc", NF90_CLOBBER, ncid)

  call nf95_def_dim(ncid, "lat", nlat, dimid_lat)
  call nf95_def_dim(ncid, "lon", nlon + 2 * copy, dimid_lon)

  call nf95_def_var(ncid, "lat", NF90_FLOAT, dimid_lat, varid_lat)
  call nf95_put_att(ncid, varid_lat, "standard_name", "latitude")
  call nf95_put_att(ncid, varid_lat, "units", "degrees_north")

  call nf95_def_var(ncid, "lon", NF90_FLOAT, dimid_lon, varid_lon)
  call nf95_put_att(ncid, varid_lon, "standard_name", "longitude")
  call nf95_put_att(ncid, varid_lon, "units", "degrees_east")

  call nf95_def_var(ncid, "extr_map", NF90_INT, [dimid_lon, dimid_lat], &
       varid_extr_map)

  call nf95_enddef(ncid)

  if (periodic) then
     call nf95_put_var(ncid, varid_lon, &
          [longitude(nlon) - 360., longitude, longitude(1) + 360.])
  else
     call nf95_put_var(ncid, varid_lon, longitude)
  end if
  
  call nf95_put_var(ncid, varid_lat, latitude)
  call nf95_put_var(ncid, varid_extr_map, extr_map)

  call nf95_close(ncid)
  print *, 'Created file "test_local_extrema.nc".'

end program test_local_extrema
